/* for using/animated-brushes.xml */
table.simplelist {
	width: auto;
}
/* add background since older style */
.guiicon {
	padding: 0.3em 0.13em 0.0em 0.13em;
	border: 1.6px solid darkgray;
	-moz-border-radius: 0.2em;
	background-color: lightgray;
}

/* cancel the default slanted style, and add background */
.guibutton {
	padding: 0.13em 0.13em;
	border: 1.6px solid darkgray;
	background-color: lightgray;
	-moz-border-radius: 0.2em;
	color: black;
	font-style: normal;
	font-weight: inherit;
}

/* cancel the default slanted style, but add underline */
.emphasis
{
	font-style: normal;
	text-decoration: underline;
	font-face: sans-serif;
}
