/* some style elements copied from wgo.css from http://www.gimp.org */
body,html {
    margin: 0;
    padding: 0;
    font-family: sans-serif;

}
body {
    background-image: url("Layout/bgpattern.png");
}

hr {
  border: 0;
}

a {
  background: transparent;
  color: #17457c;
  text-decoration: none;
  font-weight: bold;
}

a:hover {
  background: transparent;
  color: #991e1e;
  text-decoration: underline;
  font-weight: bold;
}

a img {
  border: 0;
}

h1 {
  background-color: #f5f5f5;
  color: black;
  border: 1px solid #d0d0d0;
  padding: 5px;
  margin-bottom: 30px;
  font-weight: bold;
  font-size: 110%;
  text-align: center;
}

/* Subtitle header. */
h2 {
  font-weight: bold;
  clear: both;
  font-size: 110%;
}

/* Another subtitle using another font-size then the subtitle. */
h3 {
  font-weight: bold;
  clear: both;
  font-size: 100%;
}

p {
  padding: 0 1em;
  line-height: 1.5em;
}

div + p {
  text-indent: 1em;
  clear: both;
}

li p {
  line-height: normal;
}

dt {
  text-decoration: underline;
  margin-left: 1em;
  margin-top: 1em;
}

dd {
  margin-right: 1em;
}

.webpage {
  width: 50em;
  margin: 0 auto;
  border: 0.4em solid white;
  border-top: 0;
  border-bottom: 0;
  background-color: white;
}

.section {
  padding: 0 2em;
  margin-bottom: 4em;
}

.note {
  background: #deffce url("Layout/note.png") no-repeat left center;
  padding: 0.3em;
  border: 1px solid green;
}

.note p {
   margin-left: 3em;
}

.note h3 {
   padding: 0;
   margin: 0;
   color: green;
}

.variablelist .itemizedlist ul li {
   list-style-type: none;
   float: left;
   width: 50%;
}

.variablelist .itemizedlist ul li p {
   padding-left: 1.3em;
   display: inline;
}

.cs {
   background: url("Layout/cz.png") no-repeat left center;
}

.de {
   background: url("Layout/de.png") no-repeat left center;
}
.en {
   background: url("Layout/gb.png") no-repeat left center;
}
.zh_CN {
   background: url("Layout/vn.png") no-repeat left center;
}
.fr {
   background: url("Layout/fr.png") no-repeat left center;
}
.it {
   background: url("Layout/it.png") no-repeat left center;
}
.nl {
   background: url("Layout/nl.png") no-repeat left center;
}
.sv {
   background: url("Layout/sv.png") no-repeat left center;
}
.hr {
   background: url("Layout/hr.png") no-repeat left center;
}
.es {
   background: url("Layout/es.png") no-repeat left center;
}
.no {
   background: url("Layout/no.png") no-repeat left center;
}
.ko {
   background: url("Layout/ko.png") no-repeat left center;
}
.ru {
   background: url("Layout/ru.png") no-repeat left center;
}
.pl {
   background: url("Layout/pl.png") no-repeat left center;
}
.ja {
   background: url("Layout/ja.png") no-repeat left center;
}
.el {
   background: url("Layout/el.png") no-repeat left center;
}
.fi {
   background: url("Layout/fi.png") no-repeat left center;
}
.da {
   background: url("Layout/da.png") no-repeat left center;
}
.sl {
   background: url("Layout/sl.png") no-repeat left center;
}
.pt_BR {
   background: url("Layout/pt_BR.png") no-repeat left center;
}

#Header {
    height: 8em;
    padding: 0 1em;
    margin: 0;
    background-color: #d6ba00;
    border-bottom: 1px solid black;
}

#Header img.logo {
    float: left;
    margin: 1em;
}

#Header img.wilber {
  float: right;
}


#Searchform {
  float: right;
}

#Searchform input.searchinput {
  border: 1px inset black;
  width: 15em;
}

#TopNavigation {
  padding: 0.2em;
  background-color: #17457c;
}

#TopNavigation a {
  color: white;
}

.LanguageSelection, 
.navhead {
  display: inline;
  padding: 0;
  padding-left: 0.2em;
}

.LanguageSelection li,
.navhead li {
  display: inline;
  text-align: right;
}

.navhead li {
  background-color: #17457c;
  margin: 0;
  padding: 0.2em 1em;
  border-right: 1px solid white;
}

.navhead li a {
  color: white;
}

.navhead li:hover {
  background-color: #376aa0;
}

#Box {
    border-top: 1px solid black;
}

#Content {
  margin-left: 16em;
}

#Content ol {
    margin-left: 3em;
}

#Content table.authors {
    width: 100%;
}

#Content table.authors h2 {
    border-bottom: 1px dotted #d6ba00;
    display: inline;
}

#Content table.authors tr td {
    vertical-align: top;
}

#Content table.authors dl dt {
    margin-top: 0.5em;
    letter-spacing: 0.1em;
}

#Content table.authors ul {
    list-style: none;
}

.navfoot {
  clear: both;
  background: #f5f5f5;
  color: black;
  border-top: 1px solid #d0d0d0;
  padding: 0;
  font-weight: bold;
  text-align: center;
}

.footcopy {
  font-size: small;
}

.warning {
  border: 1px solid red;
  background-color: #ffd6d6;
  width: 18em;
  padding: 0.4em;
  margin: 0 auto;
}

.Flattr {
  float: right;
}
