/* some style elements copied from wgo.css from http://www.gimp.org */
body,html {
    margin: 0;
    padding: 0;
    font-family: sans-serif;
}

a {
  background: transparent;
  color: #17457c;
  text-decoration: none;
  font-weight: bold;
}

a:hover {
  background: transparent;
  color: #991e1e;
  text-decoration: underline;
  font-weight: bold;
}

a img {
  border: 0;
}

h1 {
  background-color: #f5f5f5;
  color: black;
  border: 1px solid #d0d0d0;
  padding: 5px;
  margin-bottom: 30px;
  font-weight: bold;
  font-size: 110%;
  text-align: center;
}

/* Subtitle header. */
h2 {
  font-weight: bold;
  clear: both;
  font-size: 110%;
}

/* Another subtitle using another font-size then the subtitle. */
h3 {
  font-weight: bold;
  clear: both;
  font-size: 100%;
}

p {
  margin: 0 2em;
}

dt {
  text-decoration: underline;
  margin-left: 1em;
  margin-top: 1em;
}

dd {
  margin-right: 1em;
}

#Header {
    height: 8em;
    padding: 0 1em;
    margin: 0;
    background-color: #d6ba00;
    border-bottom: 1px solid black;
}

#Header img.logo {
    float: left;
    margin: 1em;
}

#Header img.wilber {
  float: right;
}


#Searchform {
  display: inline;
  float: right;
}

#Searchform input.searchinput {
  border: 1px inset black;
}

#TopNavigation {
  padding: 0.2em;
  background-color: #17457c;
}

#TopNavigation a {
  color: white;
}

.LanguageSelection, 
.QuickAccess {
  display: inline;
  padding: 0;
  padding-left: 0.2em;
}

.LanguageSelection li,
.QuickAccess li {
  display: inline;
  text-align: right;
}

.QuickAccess li {
  background-color: #17457c;
  margin: 0;
  padding: 0.2em 1em;
  border-right: 1px solid white;
}

.QuickAccess li a {
  color: white;
}

.QuickAccess li:hover {
  background-color: #376aa0;
}

.LanguageSelection {
  margin-left: 10em;
}

#Box {
    border-top: 1px solid black;
}

#LeftMenu {
    float: left;
    margin-right: 2em;
    margin-bottom: 0;
    background-color: #f5f5f5;
    border: 1px solid #d0d0d0;
    font-size: 80%;
    width: 20em;
}

#LeftMenu h1 {
  border: 0;
  border-bottom: 1px solid #d0d0d0;
}

#LeftMenu ul {
  padding-left: 1em;
  list-style: none;
}

#Content {
  margin-left: 16em;
}

#Content ol {
    margin-left: 3em;
}

#Content table.authors {
    width: 100%;
}

#Content table.authors h2 {
    border-bottom: 1px dotted #d6ba00;
    display: inline;
}

#Content table.authors tr td {
    vertical-align: top;
}

#Content table.authors dl dt {
    margin-top: 0.5em;
    letter-spacing: 0.1em;
}

#Content table.authors ul {
    list-style: none;
}

#Footer {
  clear: both;
  background: #f5f5f5;
  color: black;
  border-top: 1px solid #d0d0d0;
  padding: 0;
  padding-top: 0.5em;
  font-weight: bold;
  text-align: center;
}

#Copyright {
  margin-top: 0.5em;
  padding: 0.5em 0;
  background-color: white;
  border-top: 1px solid #d0d0d0;
}

.warning {
  border: 1px solid red;
  background-color: #ffd6d6;
  width: 18em;
  padding: 0.4em;
  margin: 0 auto;
}

