.caption {
	font-size : smaller;
	font-style : italic
}
pre.programlisting {
	font-size : 100%;
	font-weight : bold;
	color : Blue;
}
.guimenu,.guisubmenu,.guimenuitem,.guilabel {
	background-color: transparent;
	font-weight : bold;
	font-size : 0.9em;
	color : inherit;
}
.guibutton {
	font-size : 0.9em;
	border : thin inset #6699FF;
	-moz-border-radius: 0.3em;
	border-radius: 0.3em;
	padding-left : 0.3em;
	padding-right : 0.4em;
}
.guilabel {
	border : thin solid;
	border-color : #666666;
	-moz-border-radius: 0.3em;
	-webkit-border-radius: 0.3em;
	border-radius: 0.3em;
}