/*
   This file is part of gimp-help-2. It is copyrighted under the GNU
   General Public License Version 2 which you can get from
   http://www.gnu.org
*/
  
body
{
    color: white;
    /* red background if no image */
    background: #800000 url(images/draft.png);
}
h1,h2,h3,h4,h5,h6 {
  color: white;
}
